import pandas as pd
from scipy.stats import pearsonr
from sklearn import svm
from sklearn.metrics import make_scorer, mean_absolute_error, mean_squared_error
from sklearn.model_selection import train_test_split, GridSearchCV
import numpy as np
import pickle
from argparse import ArgumentParser

np.random.seed(0)


def load_and_structure_dataset(filename: str):
    df = pd.read_csv(filename)
    return df['sEXT'].values.tolist(), df['sAGR'].values.tolist(), df['sNEU'].values.tolist(), df['sOPN'].values.tolist(), df['sCON'].values.tolist()

def process_liwc(filename: str):
    df = pd.read_csv(filename)
    features = df.values.tolist()
    for i in range(len(features)):
        features[i].pop(0)
    for res in features:
        res[1:] = [float(r.replace(',', '.')) for r in res[1:]]
    return features

def predict(model, X_test, y_test):
    print("Predictions for model ...")
    y_pred = model.predict(X_test)
    print("MSE: " + str(mean_squared_error(y_test, y_pred)))
    print("MAE: " + str(mean_absolute_error(y_test, y_pred)))
    print("Pearson: " + str(pearsonr(y_test, y_pred)[0]))

def custom_scorer(model, X, y):
  y_pred = model.predict(X)
  y_true = y
  return pearsonr(y_true, y_pred)[0]
  
grid_scorer = make_scorer(custom_scorer, greater_is_better=True)

def train_model(X_train, y_train, param):
    clf_svm = svm.SVR()
    grid = GridSearchCV(clf_svm, param, refit=True, verbose=3, scoring=custom_scorer)
    grid.fit(X_train, y_train)
    return grid

def main(args):
    use_idun = args.idun
    save_model = args.savemodel
    train_size = 0.9

    param_grid = {
    'C': [0.1, 0.5, 1, 5, 10, 15, 20],
    'kernel': ['rbf', 'sigmoid'],
    'epsilon': [0.1, 0.2, 0.5, 0.8],
    }

    data_path = '/cluster/home/larsmagu/masteroppgave/data/dataset/personality_d.csv' if use_idun else 'personality_d.csv'
    data_path_liwc = '/cluster/home/larsmagu/masteroppgave/data/dataset/personality_d_LIWC.csv' if use_idun else 'personality_d_LIWC.csv'
    y_ext, y_agr, y_neu, y_opn, y_con = load_and_structure_dataset(data_path)
    extracted_features = process_liwc(data_path_liwc)

    X_train_ext, X_test_ext, y_train_ext, y_test_ext = train_test_split(extracted_features, y_ext, train_size = train_size, random_state=42)
    X_train_agr, X_test_agr, y_train_agr, y_test_agr = train_test_split(extracted_features, y_agr, train_size = train_size, random_state=42)
    X_train_neu, X_test_neu, y_train_neu, y_test_neu = train_test_split(extracted_features, y_neu, train_size = train_size, random_state=42)
    X_train_opn, X_test_opn, y_train_opn, y_test_opn = train_test_split(extracted_features, y_opn, train_size = train_size, random_state=42)
    X_train_con, X_test_con, y_train_con, y_test_con = train_test_split(extracted_features, y_con, train_size = train_size, random_state=42)

    model_path = '/cluster/home/larsmagu/masteroppgave/model_training/trained_models/svm_liwc'

    print("Training EXTRAVERSION....")
    model_ext = train_model(X_train_ext, y_train_ext, param_grid)
    print("Training AGREEABLENESS...")
    model_agr = train_model(X_train_agr, y_train_agr, param_grid)
    print("Training NEUROTICISM...")
    model_neu = train_model(X_train_neu, y_train_neu, param_grid)
    print("Training OPENNESS...")
    model_opn = train_model(X_train_opn, y_train_opn, param_grid)
    print("Training CONSCIENTIOUSNESS")
    model_con = train_model(X_train_con, y_train_con, param_grid)

    if save_model and use_idun:
        with open(model_path + '/model_ext_prediction.pkl', 'wb') as f:
            pickle.dump(model_ext, f)
        with open(model_path + '/model_agr_prediction.pkl', 'wb') as f:
            pickle.dump(model_agr, f)
        with open(model_path + '/model_neu_prediction.pkl', 'wb') as f:
            pickle.dump(model_neu, f)
        with open(model_path + '/model_opn_prediction.pkl', 'wb') as f:
            pickle.dump(model_opn, f)
        with open(model_path + '/model_con_prediction.pkl', 'wb') as f:
            pickle.dump(model_con, f)

    # Print prediction for models
    print("------=PREDICTIONS FOR MODELS=------")
    predict(model_ext, X_test_ext, y_test_ext)
    predict(model_agr, X_test_agr, y_test_agr)
    predict(model_neu, X_test_neu, y_test_neu)
    predict(model_opn, X_test_opn, y_test_opn)
    predict(model_con, X_test_con, y_test_con)

    print("------=BEST PARAMETERS FOR MODELS=------")
    print("Parameters EXTRAVERSION: " + str(model_ext.best_params_))
    print("Parameters AGREEABLENESS: " + str(model_agr.best_params_))
    print("Parameters NEUROTICISM: " + str(model_neu.best_params_))
    print("Parameters OPENNESS: " + str(model_opn.best_params_))
    print("Parameters CONSCIENTIOUSNESS: " + str(model_con.best_params_))

if __name__ == "__main__":
    print('Starting ...')

    parser = ArgumentParser()
    parser.add_argument("--idun", default=False, type=bool)
    parser.add_argument("--savemodel", default=True, type=bool)
    args = parser.parse_args()
    main(args)
    print('Complete')

















